
/*
 * $Id: StrIdentifier.h,v 1.2 1999/09/07 17:47:22 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */


#ifndef STRING_IDENTIFIER_H
#define STRING_IDENTIFIER_H

#include <ctype.h>
#include "Array.h"
#include "gadgets.h"


// StrIdentifier is a String->int map.
//
// StrIdentifier look-up method checks if a given buffer starts
// with one of the "known" strings. If it does, an "id" of the
// corresponding string is returned.
// No "known" string can be prefix of the other -- we do not
// want to search for the longest prefix and such...
// A buffer is not expected to be zero-terminated.

// The search algorithm is very efficient. There is probably
// no optimal static (no caching, re-grouping) algorithm.

// note: the search tree does not de-allocate its parts on destruction

class StrIdentifier; // the user interface class, declared last

class StrIdTable;
class String;

// a node on the search tree (leaf or not)
class StrIdNode {
	public:
		StrIdNode();

		bool dead() const { return theId == 0; }
		bool leaf() const { return theId > 0; }
		int id() const { return theId; }

		int lookup(const char *buf, int len) const;
		void add(String *str, int id, int myLevel);

		void optimize();

		void report(ostream &os, int myLevel) const;

	protected:
		union {
			StrIdTable *theTab; // lookup table (intermediate nodes)
			String *theStr;     // known-string (leaf nodes)
		} u;
		int theId;              // id > 0 => leaf, id < 0 => intermed
};


// collection of nodes indexed by a char at a given search pos
class StrIdTable {
	public:
		StrIdTable(int aPos);

		int lookup(const char *buf, int len) const;
		void add(String *str, int id);

		void optimize();
		bool single(StrIdNode &n);

		void report(ostream &os, int myLevel) const;

	protected:
		inline int lookupIdx(const String &str) const;
		// returns -1 if len <= theLookupPos
		inline int lookupIdx(const char *buf, int len) const;
		
	protected:
		Array<StrIdNode> theNodes;
		int theLookupPos;
};


// user interface: build(add everything), optimize, look-up.
class StrIdentifier {
	public:
		StrIdentifier();
		~StrIdentifier();

		int add(const String &str);
		int lookup(const char *buf, int len) const;
		const String &string(int id) { return *theStrings[id]; }

		// call this after all additions
		void optimize();

		// prints a search tree
		void report(ostream &os) const;

	protected:
		Array<String *> theStrings; // reverse index (id -> str)
		StrIdNode theHead;
		int theLastId;

		bool isLocked; // additions are fatal, set in optimize()
};


/* inlined methods */

inline 
int StrIdTable::lookupIdx(const String &str) const {
	return str.len() > theLookupPos ?
		xord(toupper(str[theLookupPos])) : theLookupPos;
}

inline
int StrIdTable::lookupIdx(const char *buf, int len) const {
	return len > theLookupPos ?
		xord(toupper(buf[theLookupPos])) : -1;
}

#endif
